@@ -1,4 +1,4 @@
-context("fill_na() & count_gaps() for a tsibble")
+context("fill_gaps() & count_gaps() for a tsibble")
 
 idx_day <- seq.Date(ymd("2017-01-01"), ymd("2017-01-20"), by = 4)
 dat_x <- tibble(
@@ -7,33 +7,51 @@ dat_x <- tibble(
 )
 
 test_that("a tbl_df/data.frame", {
-  expect_error(fill_na(dat_x), "data.frame")
+  expect_error(fill_gaps(dat_x), "data.frame")
 })
 
 test_that("unknown interval", {
   tsbl <- as_tsibble(dat_x[1, ], index = date)
-  expect_error(fill_na(tsbl), "data of unknown interval.")
-  expect_error(count_gaps(tsbl), "data of unknown interval.")
+  expect_identical(fill_gaps(tsbl), tsbl)
+  expect_equal(count_gaps(tsbl)$.n, 0)
+  expect_equal(has_gaps(tsbl), tibble(".gaps" = FALSE))
 })
 
 test_that("an irregular tbl_ts", {
   tsbl <- as_tsibble(dat_x, index = date, regular = FALSE)
-  expect_error(fill_na(tsbl), "irregular")
+  expect_error(fill_gaps(tsbl), "irregular")
   expect_error(count_gaps(tsbl), "irregular")
+  expect_error(has_gaps(tsbl), "irregular")
 })
 
 test_that("a tbl_ts without implicit missing values", {
   tsbl <- as_tsibble(dat_x, index = date)
-  expect_identical(fill_na(tsbl), tsbl)
-  ref_tbl <- tibble(from = NA, to = NA, n = 0L)
+  expect_identical(fill_gaps(tsbl), tsbl)
+  ref_tbl <- tibble(.from = NA, .to = NA, .n = 0L)
   expect_identical(count_gaps(tsbl), ref_tbl)
 })
 
+daylight <- pedestrian %>%
+  filter(
+    Sensor == "Birrarung Marr",
+    Date == lubridate::ymd("20151004", tz = "Australia/Melbourne")
+  )
+standard <- pedestrian %>%
+  filter(
+    Sensor == "Birrarung Marr",
+    Date == lubridate::ymd("20150405", tz = "Australia/Melbourne")
+  )
+
+test_that("daylight saving", {
+  expect_identical(NROW(fill_gaps(daylight)), 23L)
+  expect_identical(NROW(fill_gaps(standard)), 25L)
+})
+
 dat_y <- dat_x[c(1:3, 5), ]
 tsbl <- as_tsibble(dat_y, index = date)
 
 test_that("a tbl_ts of 4 day interval with no replacement", {
-  full_tsbl <- fill_na(tsbl)
+  full_tsbl <- fill_gaps(tsbl)
   expect_identical(dim(full_tsbl), c(5L, 2L))
   expect_equal(
     as_tibble(full_tsbl[4, ]),
@@ -42,7 +60,7 @@ test_that("a tbl_ts of 4 day interval with no replacement", {
 })
 
 test_that("a tbl_ts of 4 day interval with value replacement", {
-  full_tsbl <- fill_na(tsbl, value = 0)
+  full_tsbl <- fill_gaps(tsbl, value = 0)
   expect_equal(
     as_tibble(full_tsbl[4, ]),
     tibble(date = ymd("2017-01-13"), value = 0)
@@ -50,11 +68,11 @@ test_that("a tbl_ts of 4 day interval with value replacement", {
 })
 
 test_that("a tbl_ts of 4 day interval with bad names", {
-  expect_error(fill_na(tsbl, value1 = value), "Can't find column")
+  expect_error(fill_gaps(tsbl, value1 = value), "Can't find column")
 })
 
 test_that("a tbl_ts of 4 day interval with function replacement", {
-  full_tsbl <- fill_na(tsbl, value = sum(value))
+  full_tsbl <- fill_gaps(tsbl, value = sum(value))
   expect_equal(
     as_tibble(full_tsbl[4, ]),
     tibble(date = ymd("2017-01-13"), value = sum(tsbl$value))
@@ -69,8 +87,8 @@ dat_x <- tibble(
 dat_y <- dat_x[c(2:8, 10), ]
 tsbl <- as_tsibble(dat_y, key = id(group), index = date)
 
-test_that("fill_na() for corner case", {
-  expect_identical(fill_na(tsbl[1:5, ]), tsbl[1:5, ])
+test_that("fill_gaps() for corner case", {
+  expect_identical(fill_gaps(tsbl[1:5, ]), tsbl[1:5, ])
 })
 
 tourism <- tourism %>%
@@ -78,16 +96,20 @@ tourism <- tourism %>%
   slice(1:10) %>%
   ungroup()
 
-test_that("fill_na() for yearquarter", {
+test_that("fill_gaps() for yearquarter", {
   full_tsbl <- tourism %>%
-    fill_na()
+    fill_gaps()
   expect_is(full_tsbl$Quarter, "yearquarter")
 })
 
-test_that("fill_na() for a grouped_ts", {
+test_that("fill_gaps() for a grouped_ts", {
   full_tsbl <- tsbl %>%
     group_by(group) %>%
-    fill_na(value = sum(value), .full = TRUE)
+    fill_gaps(value = sum(value), .full = TRUE)
+  expect_error(
+    tourism %>% group_by(Quarter) %>% fill_gaps(Trips = sum(Trips)),
+    "Replacement"
+  )
   expect_identical(dim(full_tsbl), c(10L, 3L))
   expect_equal(
     as_tibble(full_tsbl[c(1, 9), ]),
@@ -101,7 +123,7 @@ test_that("fill_na() for a grouped_ts", {
 
 test_that("fill.tbl_ts(.full = TRUE)", {
   full_tsbl <- tsbl %>%
-    fill_na(.full = TRUE) %>%
+    fill_gaps(.full = TRUE) %>%
     group_by(group) %>%
     tidyr::fill(value, .direction = "up")
   expect_equal(
@@ -116,7 +138,7 @@ test_that("fill.tbl_ts(.full = TRUE)", {
 
 test_that("fill.tbl_ts(.full = FALSE)", {
   full_tsbl <- tsbl %>%
-    fill_na() %>%
+    fill_gaps() %>%
     group_by(group) %>%
     tidyr::fill(value, .direction = "up")
   expect_equal(
@@ -129,38 +151,43 @@ test_that("fill.tbl_ts(.full = FALSE)", {
   )
 })
 
-test_that("count_gaps.tbl_ts()", {
-  expect_equal(
-    count_gaps(tsbl),
-    tibble(from = NA, to = NA, n = 0L)
-  )
-})
-
-test_that("count_gaps.grouped_ts(.full = TRUE)", {
-  full_tbl <- tsbl  %>% group_by(group) %>% count_gaps(.full = TRUE)
+test_that("count_gaps.tbl_ts(.full = TRUE)", {
+  full_tbl <- tsbl  %>% count_gaps(.full = TRUE)
   expect_equal(
     full_tbl,
     tibble(
       group = c("a", "b"),
-      from = c(ymd("2017-01-01"), ymd("2017-01-13")),
-      to = c(ymd("2017-01-01"), ymd("2017-01-13")),
-      n = c(1L, 1L)
+      .from = c(ymd("2017-01-01"), ymd("2017-01-13")),
+      .to = c(ymd("2017-01-01"), ymd("2017-01-13")),
+      .n = c(1L, 1L)
     )
   )
 })
 
-test_that("count_gaps.grouped_ts(.full = FALSE)", {
-  full_tbl <- tsbl %>% group_by(group) %>% count_gaps()
-  a <- tibble(group = "a", from = NA, to = NA, n = 0L)
+test_that("count_gaps.tbl_ts(.full = FALSE)", {
+  full_tbl <- tsbl %>% count_gaps()
+  a <- tibble(group = "a", .from = NA, .to = NA, .n = 0L)
   b <- tibble(
     group = "b",
-    from = ymd("2017-01-13"),
-    to = ymd("2017-01-13"),
-    n = 1L
+    .from = ymd("2017-01-13"),
+    .to = ymd("2017-01-13"),
+    .n = 1L
   )
   expect_equal(full_tbl, dplyr::bind_rows(a, b))
 })
 
+harvest <- tsibble(
+  year = c(2010, 2011, 2013, 2011, 2012, 2013),
+  fruit = rep(c("kiwi", "cherry"), each = 3),
+  kilo = sample(1:10, size = 6),
+  key = id(fruit), index = year
+)
+
+test_that("has_gaps()", {
+  expect_equal(has_gaps(harvest)$.gaps, c(FALSE, TRUE))
+  expect_equal(has_gaps(harvest, .full = TRUE)$.gaps, c(TRUE, TRUE))
+})
+
 test_that("Error in gaps()", {
   expect_error(gaps(x = 1:4, y = 1:3), "must not be greater than")
 })
@@ -168,7 +195,7 @@ test_that("Error in gaps()", {
 test_that("seq_generator()", {
   x <- nanotime::nanotime("1970-01-01T00:00:00.000000001+00:00") + c(0:3, 5:9)
   expect_length(seq_generator(x), 10)
-  y <- structure("x", class = "xxx")
+  y <- structure(c("x", "y"), class = "xxx")
   pull_interval.xxx <- function(x) {init_interval(unit = 1)}
-  expect_error(seq_generator(y), "defined")
+  expect_error(seq_generator(y, pull_interval(y)), "defined")
 })
