@@ -13,6 +13,8 @@ test_that("units_since()", {
   expect_equal(units_since(yw), 0:2)
   expect_equal(units_since(ym), 0:2)
   expect_equal(units_since(yq), 0:2)
+  expect_equal(units_since(year(yq)), rep(0, 3))
+  expect_equal(units_since(1:3), 1:3)
 })
 
 test_that("diff()", {
@@ -70,7 +72,7 @@ test_that("some S3 methods for yearweek, yearmonth & yearquarter", {
 })
 
 test_that("unsupported class for index functions", {
-  expect_error(yearweek(seq(2010, 2012, by = 1 / 52)), "coerce the numeric")
+  expect_error(yearweek(seq(2010, 2012, by = 1 / 52)), "handle the numeric")
 })
 
 xx <- make_datetime(2018, 1, 1, 0)
